/*****************************************************************************
* File Name: ChargerProtocol.c
*
* Version: 1.0
*
* Description:
*   The Charger Protocol file for the BatteryCharger project.
*   Contains an Constant-Current Constant- Voltage
*   based method to charge single cell Li-ion batteries.
*
******************************************************************************
* Copyright (C) 2012, Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/


#include "LiIonCharger.h"

/* To find Row number by dividing the address by 256 */
#define ADDRESS_TO_ROW(A) (A / SIZEOF_FLASH_ROW)

/* To find address of next row */
#define ROW_INCREMENT SIZEOF_FLASH_ROW

/* Rough estimate of scaling factor of potential divider connected across battery */
#define Fixed_Cal 0.4425

/* Known voltage fed externally for calibration */
#define KNOWN_VOLTAGE 4.2 /* 4.2V */

/* State machine variables */
uint8 BatteryState=BATTERY_INIT;
uint8 ErrorState=ER_NO_ERROR;
uint8 TimerState;


/* PWM duty cycle */
uint16 PWMDuty=1000;

/* Profile Params */
uint16 ActivationStartVoltage;
uint16 RapidChargeStartVoltage;
uint16 FullChargeVoltage;
uint16 RechargeVoltage;
uint8 TerminationCurrent;
uint8 EmergStopCurrent;
uint16 EmergStopVoltage;
uint8 ActivationCurrent;
uint16 RapidChargeCurrent;
float Scale;
CYCODE const float ScaleInFlash=0.4425;

 /* Temp Buffer */
uint8 rowBuffer[SIZEOF_FLASH_ROW + SIZEOF_ECC_ROW];

/* Row address */
uint16 rowAddress1;

/* Start and end locaton of the concerned data */
uint8 rowDataOffset;
uint16 rowDataEnd;

uint16 index1, index2;

/* Array which contains value to be written to the Flash Row */
uint8 rowFinalData[SIZEOF_FLASH_ROW];

/* To find address of next row */
#define ROW_INCREMENT SIZEOF_FLASH_ROW

/* The start address of Flash in PSoC3 and PSoC5 are different.In order to make the code compatible with both 
   * PSoC3 and PSoC5, a macro is used which would specify the correct flash base according to the device used.
   * The macro is defined within a preprocessor statement which checks for the compiler used.
   * PSoC3 uses Keil_C51 and it is defined using __C51__. Else declaration for PSoC5 is used.
*/
#if (defined(__C51__))
    #define FLASH_BASE CYDEV_FLS_BASE

#else
    #define FLASH_BASE CYDEV_FLASH_BASE
    
#endif


/*******************************************************************************
* Function Name: ChargerStart()
********************************************************************************
* Summary:
*  Initialises all the components used for battery charger. ADC should be started before calling this function.
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/
void ChargerStart()
{
    uint16 Vscaled;
        
    /* Check if calibration process is activated */
    if(Pin_Calibration_Read()==0)
    {
        /* Clear LCD Display */
        LCD_ClearDisplay();
        
        /* Set the LCD position and print text "calibration" */
        LCD_Position(0,0);
        LCD_PrintString("Calibration.. ");
    
              
        /* Connect the Mux to read battery voltage. During this time, known voltage should be provided in place of battery  */
        /* In this project 4.2V is provided */
        AMux_ADC_Select(BAT_VOLTAGE);  
        
        /* Read the battery voltage */
        Vscaled=ReadADC();
        
        /* Calculate scaling factor */
        Scale=(Vscaled*0.0000625)/KNOWN_VOLTAGE;
        
        /* Stote the scaling factor in flash */    
        StoreScaleFactor();
       
        /* Give delay of 1sec */
        CyDelay(1000);
                             
    }
    else /* else use the already stored value in flash as scaling factor */
    Scale=ScaleInFlash;
    
    /* Start VDAC */
    VDAC8_RT_Start();
    
    /* Set the VDAC output thermistor measurement */
	VDAC8_RT_SetValue(0x7D);
    
    /* Get the battery charging limits */
    CalculateThresholds();
          
    /* Start IDAC and initialize its value */ 
    IDAC8_Ref_Start();
    IDAC8_Ref_SetValue(0x1E);    
        
    /* Start OC and OV comparators */
    Comp_OC_Start();
    Comp_OV_Start();
     
    /* Initialise PWM */    
    DitheredPWM_1_Start();
    DitheredPWM_1_SetPeriod(48); /* Generates 500KHz frequency */
    DitheredPWM_1_SetDuty(PWMDuty);
    /* Enable external MOSFET gate driver */
    Pin_DriverEn_Write(1);
    
    /* Start Timer_TimeOut */
	Timer_TimeOut_Init();
	
    /* Initialise Timer and ISR */
	isr_TimeOut_Start();
    Timer_Scan_Start();
    isr_ScanTime_Start();
    
    /* Reset the OC and OV logic */
    ControlReg_BlockComp_Write(0);
    ControlReg_BlockComp_Write(1);
    
    /* Enable isr_Fault */
    isr_Fault_Start();
    PWM_Sync_Start();
    
}


/*******************************************************************************
* Function Name: StoreScaleFactor()
********************************************************************************
* Summary:
*  This function stores the "Scale" variable value in flash
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/

void StoreScaleFactor()
{
     
    rowAddress1 = (uint16)((uint32)(&ScaleInFlash));
    
    /* Calculate the row number from the address, which is the remainder of address/256 (max row size) */
    rowDataOffset = rowAddress1 % SIZEOF_FLASH_ROW;

    /* Calculate the starting address of row */
    rowAddress1 = rowAddress1 - rowDataOffset;
    
    /* Calculate the location at which the concerned data ends */
    if ((uint16)rowDataOffset + sizeof(ScaleInFlash) >= SIZEOF_FLASH_ROW) 
    {
        rowDataEnd = (uint16)SIZEOF_FLASH_ROW;
    }
    else
    {
        rowDataEnd = rowDataOffset + sizeof(ScaleInFlash);
    }
    
    for(index1 = 0, index2=0; index1 < SIZEOF_FLASH_ROW; index1++)
    {
        /* Modify only the sine wave lookup table elements */
        if ((index1 >= rowDataOffset) && (index1 < rowDataEnd))
        {
            rowFinalData[index1] = *((uint8 *)(&Scale) + (uint8)index2);
            index2++;
            
        }
        else
        {
            /* Save the other elements of the flash row in the same position as they were */
            rowFinalData[index1] = CY_GET_XTND_REG8((const void *)(FLASH_BASE + rowAddress1 + index1));
        }
    }
    
    /* Must get temperature and set a temp buffer first. */
    if(!CySetTemp() && !CySetFlashEEBuffer(rowBuffer))
    {
        /* Erase and Program the row. */
        CyWriteRowData(0, ADDRESS_TO_ROW(rowAddress1), rowFinalData);
    }
    
    
    
}

/*******************************************************************************
* Function Name: CalculateThresholds()
********************************************************************************
* Summary:
*  Based on the scaling factor, this function calculates various thresholds of charging parameters
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/
void CalculateThresholds()
{
    float factor;
    
    factor=Scale/ADC_MR*ADC_RS;
    ActivationStartVoltage=(uint16) (ACT_START_VOLTAGE* factor);
    RapidChargeStartVoltage=(uint16)(RAPID_CHARGE_START_VOLTAGE*factor);
    FullChargeVoltage=(uint16) (FULL_CHARGE_VOLTAGE*factor);
    RechargeVoltage=(uint16) (RECHARGE_VOLTAGE*factor);
    EmergStopVoltage=(uint16) (EMERGENCY_CHARGE_STOP_VOLTAGE*factor);
   
}



/*******************************************************************************
* Function Name: UpdateBatteryState()
********************************************************************************
* Summary:
*  updates the state machine which implements the charging algorithm
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/
void UpdateBatteryState()
{
    if(TBat > TCOLD_STOP && TBat < THOT_STOP)
    {
        /* Temperature is within the limits */
        if(BatteryState==WAIT_FOR_TEMP)
        {
            /* Reset the battery state */
            BatteryState=BATTERY_INIT;
			
			/* Reset Timer */
			Timer_TimeOut_Stop();
			Timer_TimeOut_Init();
		    isr_TimeOut_ClearPending();
			TimerState = OFF;
            TimeOut=0;
            
            /* Enable charger */
            PWMDuty=LOLIMIT;
            DitheredPWM_1_SetDuty(PWMDuty);
            
            /* Enable external MOSFET gate driver */
            Pin_DriverEn_Write(1);
            
        }
        
        if(BatteryState == BATTERY_INIT)
        {
              
            if(VBat < ActivationStartVoltage)
            {
                /* Enter the state to supply initial activation current */
                if(TimerState==OFF)
                {
                   
                    /* Set the Time Out Limit */
                    Timer_TimeOut_WriteCounter(ACT_TIME_LIMIT);
                    Timer_TimeOut_Enable();
                    TimerState=ON;
                }
                if(TimeOut)
                {
                    /*  Time out is detected. Battery failed to charge to certain level in predefined time. 
                    Move to Battery Error State */
                    TimeOut=0;
                    TimerState=OFF;
                    
                    /* Enter into ERROR state */    
                    BatteryState=BATTERY_ERROR;
					ErrorState = ER_START_TIMEOUT;
                                   
                }
                                    
            }
            else
            if(VBat < RechargeVoltage)
            {
                if(TimerState==ON)
                {
                    /* Stop the timer */
                    Timer_TimeOut_Stop();
                    TimerState=OFF;
                    TimeOut=0;
                }
                
                /* Enter into activation state */
                BatteryState=ACTIVATION;
                
            }
            else
            if(VBat > RechargeVoltage)
            {
                if(TimerState==ON)
                {
                    /* Stop the timer */
                    Timer_TimeOut_Stop();
                    TimerState=OFF;
                    TimeOut=0;
                }
                /* Enter into Complete Charge state */
                BatteryState=CHARGE_COMPLETE;
                
                
            }
        }
        else
        if(BatteryState == ACTIVATION)
        {
            if(TimerState==OFF)
            {
                /* Set the Time Out Limit */
                Timer_TimeOut_WriteCounter(ACT_TO_RAPID_TIME_LIMIT);
                Timer_TimeOut_Enable();
                TimerState=ON;
                
            }
            if(TimeOut)
            {
                /*  Time out is detected. Battery failed to charge to certain level in predefined time. 
                    Move to Battery Error State */
                    
                TimeOut=0;
                TimerState=OFF;
                
                /* Enter into ERROR state */    
                BatteryState=BATTERY_ERROR;
				ErrorState = ER_ACTIVE_TIMEOUT;
                
            }
                
                      
            if(VBat > EmergStopVoltage || IBat > EMERGENCY_CHARGE_STOP_CURRENT)
            {
                /* Move to battery error state */
                BatteryState=BATTERY_ERROR;
				if(VBat > EmergStopVoltage)
				{
					ErrorState = ER_OVER_VOLTAGE;
				}
				if( IBat > EMERGENCY_CHARGE_STOP_CURRENT)
				{
					ErrorState = ER_OVER_CURRENT;
				}
                
            }
            else
            if(VBat > RapidChargeStartVoltage)
            {
                /* Move to Rapid Charge State */
                BatteryState=RAPID_CHARGE;
                
                Timer_TimeOut_Stop();
                TimerState=OFF;
                TimeOut=0;
                    
            }
            
        }
        else
        if(BatteryState==RAPID_CHARGE)
        {
            if(TimerState==OFF)
            {
                /* Set the Time Out Limit */
                Timer_TimeOut_WriteCounter(RAPID_CHARGE_TIME_LIMIT);
                Timer_TimeOut_Enable();
                TimerState=ON;
            }
            if(TimeOut)
            {
                /*  Time out is detected. Battery failed to charge to certain level in predefined time. 
                    Move to Battery Error State */
                    
                TimeOut=0;
                TimerState=OFF;
                /* Enter into ERROR state */    
                BatteryState=BATTERY_ERROR;
                ErrorState = ER_RAPID_TIMEOUT;
                
            }
            
            if(VBat > EmergStopVoltage || IBat > EMERGENCY_CHARGE_STOP_CURRENT)
            {
                /* Move to error state */    
                Timer_TimeOut_Stop();
                TimerState=OFF;
                BatteryState=BATTERY_ERROR;
				if(VBat > EmergStopVoltage)
            	{
					ErrorState = ER_OVER_VOLTAGE;
				}
				if(IBat > EMERGENCY_CHARGE_STOP_CURRENT)
				{
					ErrorState = ER_OVER_CURRENT;
				}
                
            }
            else
            if(VBat > FullChargeVoltage)
            {
                /* Move to constant voltage state */
                Timer_TimeOut_Stop();
                TimerState=OFF;
                TimeOut=0;
                BatteryState=CONST_VOLTAGE;
            }     
        }
        else
        if(BatteryState==CONST_VOLTAGE)
        {
            if(TimerState==OFF)
            {
                /* Set the Time Out Limit */
                Timer_TimeOut_WriteCounter(CONST_VOLTAGE_TIME_LIMIT);
                Timer_TimeOut_Enable();
                TimerState=ON;
                
            }
            if(TimeOut)
            {
                /*  Time out is detected. Battery failed to charge to certain level in predefined time. 
                    Move to Battery Error State */
                    
                TimeOut=0;
                
                /* Enter into ERROR state */    
                BatteryState=BATTERY_ERROR;
                ErrorState = ER_CV_TIMEOUT;
                
            }
            
            if(VBat > EmergStopVoltage || IBat > EMERGENCY_CHARGE_STOP_CURRENT)
            {
                /* Move to battery error state */
                Timer_TimeOut_Stop();
                TimerState=OFF;
                TimeOut=0;
                BatteryState=BATTERY_ERROR;
				if(VBat > EMERGENCY_CHARGE_STOP_VOLTAGE)
				{
					ErrorState = ER_OVER_VOLTAGE;
				}
				if(IBat > EMERGENCY_CHARGE_STOP_CURRENT)
				{
					ErrorState = ER_OVER_CURRENT;
				}
                
            }
            else
            if(IBat < TERMINATION_CURRENT)
            {
                /* Charge Complete */
                Timer_TimeOut_Stop();
                TimerState=OFF; 
                TimeOut=0;
                BatteryState=CHARGE_COMPLETE;
            }
            
        }
        else
        if(BatteryState==BATTERY_ERROR)
        {
                      
            /* shut down charger */
            PWMDuty=LOLIMIT;
            DitheredPWM_1_SetDuty(0);
            TimeOut=0;
            
            /* disable external MOSFET gate driver */
            Pin_DriverEn_Write(0);
            
            Fault = 1; 
        }
        else
        if(BatteryState==CHARGE_COMPLETE)
        {
            if(VBat > RechargeVoltage)
            {
                /* shut down charger */
                PWMDuty=LOLIMIT;
                DitheredPWM_1_SetDuty(0);
                
                /* disable external MOSFET gate driver */
                Pin_DriverEn_Write(0);
            }
            else
            {
                /* Reset the battery state machine */
                TimerState=OFF; 
                TimeOut=0;
                BatteryState=BATTERY_INIT;
                
                /* Enable charger */
                PWMDuty=LOLIMIT;
                DitheredPWM_1_SetDuty(PWMDuty);
                
                /* Enable external MOSFET gate driver */
                Pin_DriverEn_Write(1);
                                    
            }
        }
                   
    }
    else
    {
        /* Temperature is outside the limits */
            
          
        Fault = 1;
        BatteryState=WAIT_FOR_TEMP;
        ErrorState = ER_TEMP_ERROR;
               
    }
}

/*******************************************************************************
* Function Name: UpdateControlLoop()
********************************************************************************
* Summary:
*  Depending on the state of the battery, this function updates the PWM duty to follow
*  predefined charging profile
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/
void UpdateControlLoop()
{

    /* Disable comparator output/Fault capture latch*/
    ControlReg_BlockComp_Write(0);
    
    if(BatteryState==BATTERY_INIT || BatteryState==ACTIVATION)
    {
        /* Update the PWM duty to maintain predefined charging current value */
        if(IBat < (ACTIVATION_CURRENT - Curr_ActDiff))
        {
            PWMDuty = PWMDuty + PWM_ActStep;
            if(PWMDuty < HILIMIT)
            {
                Pin_DriverEn_Write(0);
                DitheredPWM_1_SetDuty(PWMDuty);
                Pin_DriverEn_Write(1);
            }
                
            else
                PWMDuty=HILIMIT;
        }
        else
        if(IBat > (ACTIVATION_CURRENT + Curr_ActDiff))
        {
            if(PWMDuty > LOLIMIT)
                PWMDuty = PWMDuty  - PWM_ActStep;
            Pin_DriverEn_Write(0);
            DitheredPWM_1_SetDuty(PWMDuty);
            Pin_DriverEn_Write(1);
        }
    }
    else
    if(BatteryState == RAPID_CHARGE)
    {
        if(IBat < (RAPID_CHARGE_CURRENT- Curr_RapidDiff))
        {
            PWMDuty = PWMDuty + PWM_RapidStep;
            if(PWMDuty < HILIMIT)
            {
                Pin_DriverEn_Write(0);
                DitheredPWM_1_SetDuty(PWMDuty);
                Pin_DriverEn_Write(1);
            }
            else
                PWMDuty=HILIMIT;
        }
        else
        if(IBat > (RAPID_CHARGE_CURRENT + Curr_RapidDiff))
        {
            if(PWMDuty > LOLIMIT)
                PWMDuty = PWMDuty - PWM_RapidStep;
                
            Pin_DriverEn_Write(0);
            DitheredPWM_1_SetDuty(PWMDuty);
            Pin_DriverEn_Write(1);
        }
    }
    else
    if(BatteryState == CONST_VOLTAGE)
    {
        /* Update the PWM duty to maintain predefined battery voltage value */
        if(VBat < (FullChargeVoltage-Volt_ConstDiff))
        {
            PWMDuty = PWMDuty + PWM_ConstStep;
            if(PWMDuty < HILIMIT)
            {
                Pin_DriverEn_Write(0);
                DitheredPWM_1_SetDuty(PWMDuty);
                Pin_DriverEn_Write(1);
            }
            else
                PWMDuty=HILIMIT;
        }
        else
        if((VBat > FullChargeVoltage) && (IBat > TERMINATION_CURRENT))
        {
            if(PWMDuty > LOLIMIT)
                PWMDuty = PWMDuty - PWM_ConstStep;
                
            Pin_DriverEn_Write(0);
            DitheredPWM_1_SetDuty(PWMDuty);
            Pin_DriverEn_Write(1); 
        }
    }
    else
    if(BatteryState & (BATTERY_ERROR | WAIT_FOR_TEMP | NO_BATTERY) )
    {
        /* Shut down PWM */
        PWMDuty = LOLIMIT;
        Pin_DriverEn_Write(0);
        DitheredPWM_1_SetDuty(0);    
        
    }
    
    /* Enable comparator output latch*/
    ControlReg_BlockComp_Write(1);
}
