/*******************************************************************************
* File Name: IDAC8_OV.c  
* Version 1.60
*
*  Description:
*    This file provides the source code to the API for the 8-bit Current 
*    DAC (IDAC8) User Module.
*
*   Note:
*     
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "IDAC8_OV.h"   

uint8 IDAC8_OV_initVar = 0u;


/*******************************************************************************
* Function Name: IDAC8_OV_Init
********************************************************************************
* Summary:
*  Initialize to the schematic state.
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
*******************************************************************************/
void IDAC8_OV_Init(void) 
{
    IDAC8_OV_CR0 = (IDAC8_OV_MODE_I | IDAC8_OV_DEFAULT_RANGE )  ;    

    /* Set default data source */
    if(IDAC8_OV_DEFAULT_DATA_SRC != 0u )    
    {
        IDAC8_OV_CR1 = (IDAC8_OV_DEFAULT_CNTL | IDAC8_OV_DACBUS_ENABLE | IDAC8_OV_DEFAULT_POLARITY) ;   
    }
    else
    {
        IDAC8_OV_CR1 = (IDAC8_OV_DEFAULT_CNTL | IDAC8_OV_DACBUS_DISABLE | IDAC8_OV_DEFAULT_POLARITY) ;   
    } 
    
    /* Set default strobe mode */
    if(IDAC8_OV_DEFAULT_STRB != 0u)
    {
        IDAC8_OV_Strobe |= IDAC8_OV_STRB_EN ;
    }
    
    /* Set default speed */
    IDAC8_OV_SetSpeed(IDAC8_OV_DEFAULT_SPEED); 
    
    /* Set proper DAC trim */
    IDAC8_OV_DacTrim();   
}


/*******************************************************************************
* Function Name: IDAC8_OV_Enable
********************************************************************************
* Summary:
*     Enable the IDAC8
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
*******************************************************************************/
void IDAC8_OV_Enable(void) 
{
    IDAC8_OV_PWRMGR |= IDAC8_OV_ACT_PWR_EN;
    IDAC8_OV_STBY_PWRMGR |= IDAC8_OV_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: IDAC8_OV_Start
********************************************************************************
* Summary:
*  Set power level then turn on IDAC8.
*
* Parameters:  
*  power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Global variables:
*  IDAC8_OV_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void IDAC8_OV_Start(void) 
{
    /* Hardware initiazation only needs to occur the first time */
    if ( IDAC8_OV_initVar == 0u)  
    {     
        IDAC8_OV_Init();
       
        IDAC8_OV_initVar = 1;
    }  
   
    /* Enable power to DAC */
    IDAC8_OV_Enable();

    /* Set default value */
    IDAC8_OV_SetValue(IDAC8_OV_DEFAULT_DATA);

}


/*******************************************************************************
* Function Name: IDAC8_OV_Stop
********************************************************************************
* Summary:
*  Powers down IDAC8 to lowest power state.
*
* Parameters:  
*   (void)
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_OV_Stop(void) 
{
   /* Disble power to DAC */
   IDAC8_OV_PWRMGR &= ~IDAC8_OV_ACT_PWR_EN;
   IDAC8_OV_STBY_PWRMGR &= ~IDAC8_OV_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: IDAC8_OV_SetSpeed
********************************************************************************
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_OV_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   IDAC8_OV_CR0 &= ~IDAC8_OV_HS_MASK ;    
   IDAC8_OV_CR0 |=  ( speed & IDAC8_OV_HS_MASK) ;    
}


/*******************************************************************************
* Function Name: IDAC8_OV_SetPolarity
********************************************************************************
* Summary:
*  Sets IDAC to Sink or Source current
*
* Parameters:  
*  Polarity: Sets the IDAC to Sink or Source
*   0x00 - Source
*   0x04 - Sink
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_OV_SetPolarity(uint8 polarity) 
{
   	IDAC8_OV_CR1 &= ~IDAC8_OV_IDIR_MASK;                /* clear polarity bit */
    IDAC8_OV_CR1 |= (polarity & IDAC8_OV_IDIR_MASK);    /* set new value */
    
    IDAC8_OV_DacTrim();
}

/*******************************************************************************
* Function Name: IDAC8_OV_SetRange
********************************************************************************
* Summary:
*  Set current range
*
* Parameters:  
*  Range:  Sets on of four valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_OV_SetRange(uint8 range) 
{
   IDAC8_OV_CR0 &= ~IDAC8_OV_RANGE_MASK ;       /* Clear existing mode */
   IDAC8_OV_CR0 |= ( range & IDAC8_OV_RANGE_MASK );  /*  Set Range  */
   IDAC8_OV_DacTrim();
}


/*******************************************************************************
* Function Name: IDAC8_OV_SetValue
********************************************************************************
* Summary:
*  Set DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_OV_SetValue(uint8 value) 
{
   IDAC8_OV_Data = value;         /*  Set Value  */
   
   /* PSOC3 silicons earlier to ES3 require a double write */
   #if (IDAC8_OV_PSOC3_ES2 || IDAC8_OV_PSOC5_ES1)
       IDAC8_OV_Data = value;
   #endif
}

/*******************************************************************************
* Function Name: IDAC8_OV_DacTrim
********************************************************************************
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*  None
*
* Return: 
*  (void) 
*
* Theory: 
*  Trim values for the IDAC blocks are stored in the "Customer Table" area in 
*  Row 1 of the Hidden Flash.  There are 8 bytes of trim data for each IDAC block.
*  The values are:
*       I Gain offset, min range, Sourcing
*       I Gain offset, min range, Sinking
*       I Gain offset, med range, Sourcing
*       I Gain offset, med range, Sinking
*       I Gain offset, max range, Sourcing
*       I Gain offset, max range, Sinking
*       V Gain offset, 1V range
*       V Gain offset, 4V range
*
* Side Effects:
*
*******************************************************************************/
void IDAC8_OV_DacTrim(void) 
{
    uint8 mode;

    mode = ((IDAC8_OV_CR0 & IDAC8_OV_RANGE_MASK) >> 1);
    
    if((IDAC8_OV_IDIR_MASK & IDAC8_OV_CR1) == IDAC8_OV_IDIR_SINK)
    {
        mode++;
    }

    IDAC8_OV_TR = CY_GET_XTND_REG8((uint8 *)(IDAC8_OV_DAC_TRIM_BASE + mode));
}


/* [] END OF FILE */
