/*******************************************************************************
* File Name: ScanTimer_PM.c  
* Version 2.0
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "ScanTimer.h"
static ScanTimer_backupStruct ScanTimer_backup;

/*******************************************************************************
* Function Name: ScanTimer_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  ScanTimer_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void ScanTimer_SaveConfig(void)
{   
    #if (!ScanTimer_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (ScanTimer_PSOC3_ES2 || ScanTimer_PSOC5_ES1)
            ScanTimer_backup.TimerUdb = ScanTimer_ReadCounter();
            ScanTimer_backup.TimerPeriod = ScanTimer_ReadPeriod();
            ScanTimer_backup.InterruptMaskValue = ScanTimer_STATUS_MASK;
            #if (ScanTimer_UsingHWCaptureCounter)
                ScanTimer_backup.TimerCaptureCounter = ScanTimer_ReadCaptureCount();        
            #endif
        #endif
        
        #if (ScanTimer_PSOC3_ES3 || ScanTimer_PSOC5_ES2)
            ScanTimer_backup.TimerUdb = ScanTimer_ReadCounter();
            ScanTimer_backup.InterruptMaskValue = ScanTimer_STATUS_MASK;
            #if (ScanTimer_UsingHWCaptureCounter)
                ScanTimer_backup.TimerCaptureCounter = ScanTimer_ReadCaptureCount();        
            #endif
        #endif
        
        #if(!ScanTimer_ControlRegRemoved)
            ScanTimer_backup.TimerControlRegister = ScanTimer_ReadControlRegister();
        #endif
    #endif
}

/*******************************************************************************
* Function Name: ScanTimer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  ScanTimer_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void ScanTimer_RestoreConfig(void) 
{
    #if (!ScanTimer_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (ScanTimer_PSOC3_ES2 || ScanTimer_PSOC5_ES1)
            ScanTimer_WriteCounter(ScanTimer_backup.TimerUdb);
            ScanTimer_WritePeriod(ScanTimer_backup.TimerPeriod);
            ScanTimer_STATUS_MASK =ScanTimer_backup.InterruptMaskValue;
            #if (ScanTimer_UsingHWCaptureCounter)
                ScanTimer_SetCaptureCount(ScanTimer_backup.TimerCaptureCounter);        
            #endif
        #endif
        
        #if (ScanTimer_PSOC3_ES3 || ScanTimer_PSOC5_ES2)
            ScanTimer_WriteCounter(ScanTimer_backup.TimerUdb);
            ScanTimer_STATUS_MASK =ScanTimer_backup.InterruptMaskValue;
            #if (ScanTimer_UsingHWCaptureCounter)
                ScanTimer_SetCaptureCount(ScanTimer_backup.TimerCaptureCounter);           
            #endif
        #endif
        
        #if(!ScanTimer_ControlRegRemoved)
            ScanTimer_WriteControlRegister(ScanTimer_backup.TimerControlRegister);
        #endif
    #endif
}


/*******************************************************************************
* Function Name: ScanTimer_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  ScanTimer_backup.TimerEnableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*    No
*
*******************************************************************************/
void ScanTimer_Sleep(void)
{
    #if(!ScanTimer_ControlRegRemoved)
        /* Save Counter's enable state */
        if(ScanTimer_CTRL_ENABLE == (ScanTimer_CONTROL & ScanTimer_CTRL_ENABLE))
        {
            /* Timer is enabled */
            ScanTimer_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            ScanTimer_backup.TimerEnableState = 0u;
        }
    #endif
    ScanTimer_Stop();
    ScanTimer_SaveConfig();
}


/*******************************************************************************
* Function Name: ScanTimer_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  ScanTimer_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void ScanTimer_Wakeup(void) 
{
    ScanTimer_RestoreConfig();
    #if(!ScanTimer_ControlRegRemoved)
        if(ScanTimer_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                ScanTimer_Enable();
        } /* Do nothing if Timer was disabled before */ 
    #endif
}


/* [] END OF FILE */
