/*******************************************************************************
* File Name: Thermistor_1.c
* Version 1.0
*
* Description:
*  This file provides the API source code for the Thermistor component.
*
* Note: none
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/
#include "Thermistor_1.h"
#include <device.h>
/******************************************************************************* 
* Function Name: Thermistor_1_Enable
********************************************************************************
*
* Summary: 
*  In Equation method, the Stainhart-Hart equation is used to obtain the temperature.
*  The digital values of the voltages across the reference resistor and the thermistor are passed to this function as parameters. 
*  These can be considered as the inputs to the component. The function returns (outputs) the temperature, based on the voltage values.
*  
*
* Parameters:  
*  Vreference is the voltage across the reference resistor. 
*  Vthermistor is the voltage across the thermistor.
*
* Return: 
*  The return value is the temperature. The return type is 32-bit integer as shown in the function prototype provided above. 
*  The value returned is the scaled version of temperature. The value is scaled up by 100 to avoid use of floating point math in the calculations. 
*  For example, the return value is 2345, when the actual temperature is 23.45 
*
* Reentrance: 
*  No
*
*******************************************************************************/
#if(Thermistor_1_IMPL ==1) 
	#include "math.h"
	int32 Thermistor_1_GetTemperature(int32 Vref, int32 VT)
	{
/* local variables */
		float LogrT;
		int32 iTemp_TR;
/* Calculate thermistor resistance from the voltages */		
		LogrT = log(Thermistor_1_Ref_Res *((float)(VT)/((float)(Vref))));
/* Calculate temperature from the resistance of thermistor using Steinhart-Hart Equation */
		iTemp_TR =(int32) (((1/(Thermistor_1_THA +(Thermistor_1_THB*LogrT)+(Thermistor_1_THC*pow(LogrT,3.0))))-273.15) * 100);
/* Return temperature */
		return(iTemp_TR);
	}
#endif

/******************************************************************************* 
* Function Name: Thermistor_1_Enable
********************************************************************************
*
* Summary: 
*  In LUT method, the look-up-table generated by the component customizer software is used to obtain the temperature.
*  The digital values of the voltages across the reference resistor and the thermistor are passed to this function as parameters. 
*  These can be considered as the inputs to the component. The function returns (outputs) the temperature, based on the voltage values.
*  
*
* Parameters:  
*  Vreference is the voltage across the reference resistor. 
*  Vthermistor is the voltage across the thermistor.
*
* Return: 
*  The return value is the temperature. The return type is 32-bit integer as shown in the function prototype provided above. 
*  The value returned is the scaled version of temperature. The value is scaled up by 100 to avoid use of floating point math in the calculations. 
*  For example, the return value is 2345, when the actual temperature is 23.45 
*
* Reentrance: 
*  No
*
*******************************************************************************/
#if (Thermistor_1_IMPL == 0)
	int32 Thermistor_1_GetTemperature(int32 Vref, int32 VT)
	{
/* local variables */
		CYCODE const uint32 Thermistor_1_LUTARR[Thermistor_1_LUTSIZE] = Thermistor_1_LUT;
		uint32 lRes_TR;
		int32 iTemp_TR;
		uint16 iLoop;
		uint8 iAccuracy;
/* Calculate thermistor resistance from the voltages */	
		lRes_TR= Thermistor_1_Ref_Res *(long)VT;
		lRes_TR = lRes_TR + (long)Vref/2;
		lRes_TR = lRes_TR / (long)Vref;
		iAccuracy = Thermistor_1_Accuracy * 100;
/* Calculate temperature from the resistance of thermistor by using the LUT */			
		for(iLoop =0; iLoop < Thermistor_1_LUTSIZE; iLoop++)
		{
			if(lRes_TR > Thermistor_1_LUTARR[iLoop])
			{
				break;
			}
		}
		iTemp_TR = Thermistor_1_MIN_TEMP * 100 + (iLoop * iAccuracy);
/* Return temperature */
		return(iTemp_TR);
	}
#endif

/* [] END OF FILE */
