/*******************************************************************************
* File Name: TimeOut_PM.c  
* Version 2.0
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "TimeOut.h"
static TimeOut_backupStruct TimeOut_backup;

/*******************************************************************************
* Function Name: TimeOut_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  TimeOut_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
* Reentrant:
*    No
*
*******************************************************************************/
void TimeOut_SaveConfig(void)
{   
    #if (!TimeOut_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (TimeOut_PSOC3_ES2 || TimeOut_PSOC5_ES1)
            TimeOut_backup.TimerUdb = TimeOut_ReadCounter();
            TimeOut_backup.TimerPeriod = TimeOut_ReadPeriod();
            TimeOut_backup.InterruptMaskValue = TimeOut_STATUS_MASK;
            #if (TimeOut_UsingHWCaptureCounter)
                TimeOut_backup.TimerCaptureCounter = TimeOut_ReadCaptureCount();        
            #endif
        #endif
        
        #if (TimeOut_PSOC3_ES3 || TimeOut_PSOC5_ES2)
            TimeOut_backup.TimerUdb = TimeOut_ReadCounter();
            TimeOut_backup.InterruptMaskValue = TimeOut_STATUS_MASK;
            #if (TimeOut_UsingHWCaptureCounter)
                TimeOut_backup.TimerCaptureCounter = TimeOut_ReadCaptureCount();        
            #endif
        #endif
        
        #if(!TimeOut_ControlRegRemoved)
            TimeOut_backup.TimerControlRegister = TimeOut_ReadControlRegister();
        #endif
    #endif
}

/*******************************************************************************
* Function Name: TimeOut_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TimeOut_backup:  Variables of this global structure are used to restore 
*  the values of non retention registers on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void TimeOut_RestoreConfig(void) 
{
    #if (!TimeOut_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC3 ES2 and PSoC5 ES1*/
        #if (TimeOut_PSOC3_ES2 || TimeOut_PSOC5_ES1)
            TimeOut_WriteCounter(TimeOut_backup.TimerUdb);
            TimeOut_WritePeriod(TimeOut_backup.TimerPeriod);
            TimeOut_STATUS_MASK =TimeOut_backup.InterruptMaskValue;
            #if (TimeOut_UsingHWCaptureCounter)
                TimeOut_SetCaptureCount(TimeOut_backup.TimerCaptureCounter);        
            #endif
        #endif
        
        #if (TimeOut_PSOC3_ES3 || TimeOut_PSOC5_ES2)
            TimeOut_WriteCounter(TimeOut_backup.TimerUdb);
            TimeOut_STATUS_MASK =TimeOut_backup.InterruptMaskValue;
            #if (TimeOut_UsingHWCaptureCounter)
                TimeOut_SetCaptureCount(TimeOut_backup.TimerCaptureCounter);           
            #endif
        #endif
        
        #if(!TimeOut_ControlRegRemoved)
            TimeOut_WriteControlRegister(TimeOut_backup.TimerControlRegister);
        #endif
    #endif
}


/*******************************************************************************
* Function Name: TimeOut_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TimeOut_backup.TimerEnableState:  Is modified depending on the enable state
*  of the block before entering sleep mode.
*
* Reentrant:
*    No
*
*******************************************************************************/
void TimeOut_Sleep(void)
{
    #if(!TimeOut_ControlRegRemoved)
        /* Save Counter's enable state */
        if(TimeOut_CTRL_ENABLE == (TimeOut_CONTROL & TimeOut_CTRL_ENABLE))
        {
            /* Timer is enabled */
            TimeOut_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            TimeOut_backup.TimerEnableState = 0u;
        }
    #endif
    TimeOut_Stop();
    TimeOut_SaveConfig();
}


/*******************************************************************************
* Function Name: TimeOut_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  TimeOut_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
* Reentrant:
*    Yes
*
*******************************************************************************/
void TimeOut_Wakeup(void) 
{
    TimeOut_RestoreConfig();
    #if(!TimeOut_ControlRegRemoved)
        if(TimeOut_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                TimeOut_Enable();
        } /* Do nothing if Timer was disabled before */ 
    #endif
}


/* [] END OF FILE */
