/*******************************************************************************
* File Name: PWM_Sync_PM.c
* Version 3.0
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#include "cytypes.h"
#include "PWM_Sync.h"

static PWM_Sync_backupStruct PWM_Sync_backup;


/*******************************************************************************
* Function Name: PWM_Sync_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*  
* Parameters:  
*  None
*
* Return: 
*  None
*
* Global variables:
*  PWM_Sync_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void PWM_Sync_SaveConfig(void) 
{
    
    #if(!PWM_Sync_UsingFixedFunction)
        #if(!PWM_Sync_PWMModeIsCenterAligned)
            PWM_Sync_backup.PWMPeriod = PWM_Sync_ReadPeriod();
        #endif /* (!PWM_Sync_PWMModeIsCenterAligned) */
        PWM_Sync_backup.PWMUdb = PWM_Sync_ReadCounter();
        #if (PWM_Sync_UseStatus)
            PWM_Sync_backup.InterruptMaskValue = PWM_Sync_STATUS_MASK;
        #endif /* (PWM_Sync_UseStatus) */
        
        #if(PWM_Sync_DeadBandMode == PWM_Sync__B_PWM__DBM_256_CLOCKS || \
            PWM_Sync_DeadBandMode == PWM_Sync__B_PWM__DBM_2_4_CLOCKS)
            PWM_Sync_backup.PWMdeadBandValue = PWM_Sync_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */
        
        #if(PWM_Sync_KillModeMinTime)
             PWM_Sync_backup.PWMKillCounterPeriod = PWM_Sync_ReadKillTime();
        #endif /* (PWM_Sync_KillModeMinTime) */
        
        #if(PWM_Sync_UseControl)
            PWM_Sync_backup.PWMControlRegister = PWM_Sync_ReadControlRegister();
        #endif /* (PWM_Sync_UseControl) */
    #endif  /* (!PWM_Sync_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: PWM_Sync_RestoreConfig
********************************************************************************
* 
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:  
*  None
*
* Return: 
*  None
*
* Global variables:
*  PWM_Sync_backup:  Variables of this global structure are used to  
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_Sync_RestoreConfig(void) 
{
        #if(!PWM_Sync_UsingFixedFunction)
            #if(!PWM_Sync_PWMModeIsCenterAligned)
                PWM_Sync_WritePeriod(PWM_Sync_backup.PWMPeriod);
            #endif /* (!PWM_Sync_PWMModeIsCenterAligned) */
            PWM_Sync_WriteCounter(PWM_Sync_backup.PWMUdb);
            #if (PWM_Sync_UseStatus)
                PWM_Sync_STATUS_MASK = PWM_Sync_backup.InterruptMaskValue;
            #endif /* (PWM_Sync_UseStatus) */
            
            #if(PWM_Sync_DeadBandMode == PWM_Sync__B_PWM__DBM_256_CLOCKS || \
                PWM_Sync_DeadBandMode == PWM_Sync__B_PWM__DBM_2_4_CLOCKS)
                PWM_Sync_WriteDeadTime(PWM_Sync_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */
            
            #if(PWM_Sync_KillModeMinTime)
                PWM_Sync_WriteKillTime(PWM_Sync_backup.PWMKillCounterPeriod);
            #endif /* (PWM_Sync_KillModeMinTime) */
            
            #if(PWM_Sync_UseControl)
                PWM_Sync_WriteControlRegister(PWM_Sync_backup.PWMControlRegister); 
            #endif /* (PWM_Sync_UseControl) */
        #endif  /* (!PWM_Sync_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: PWM_Sync_Sleep
********************************************************************************
* 
* Summary:
*  Disables block's operation and saves the user configuration. Should be called 
*  just prior to entering sleep.
*  
* Parameters:  
*  None
*
* Return: 
*  None
*
* Global variables:
*  PWM_Sync_backup.PWMEnableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void PWM_Sync_Sleep(void) 
{
    #if(PWM_Sync_UseControl)
        if(PWM_Sync_CTRL_ENABLE == (PWM_Sync_CONTROL & PWM_Sync_CTRL_ENABLE))
        {
            /*Component is enabled */
            PWM_Sync_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            PWM_Sync_backup.PWMEnableState = 0u;
        }
    #endif /* (PWM_Sync_UseControl) */

    /* Stop component */
    PWM_Sync_Stop();
    
    /* Save registers configuration */
    PWM_Sync_SaveConfig();
}


/*******************************************************************************
* Function Name: PWM_Sync_Wakeup
********************************************************************************
* 
* Summary:
*  Restores and enables the user configuration. Should be called just after 
*  awaking from sleep.
*  
* Parameters:  
*  None
*
* Return: 
*  None
*
* Global variables:
*  PWM_Sync_backup.pwmEnable:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void PWM_Sync_Wakeup(void) 
{
     /* Restore registers values */
    PWM_Sync_RestoreConfig();
    
    if(PWM_Sync_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        PWM_Sync_Enable();
    } /* Do nothing if component's block was disabled before */
    
}


/* [] END OF FILE */
