/*****************************************************************************
* File Name: SignalProcessing.c
*
* Version: 1.0
*
* Description:
*   The Signal Processing file for the BatteryCharger project.
*   Contains an Constant-Current Constant- Voltage
*   based method to charge single cell Li-ion batteries.
*
******************************************************************************
* Copyright (C) 2012, Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/

#include "LiIonCharger.h"

/* Variables */
int16 VBat,IBat, TBat;

/*******************************************************************************
* Function Name: ReadADC()
********************************************************************************
* Summary:
*  This function reads the ADC for the selected channel 
* 
*  Parameters:  None
*
*  Return: ADC value (int16)
*******************************************************************************/
int16 ReadADC()
{
    int16 ADCValue;
    
    /* Start the ADC conversion */
    ADC_DelSig_StartConvert();
    
    ADC_DelSig_IsEndConversion(ADC_DelSig_WAIT_FOR_RESULT);
    ADCValue = ADC_DelSig_GetResult16();
    
    /* Stop ADC conversion */
    ADC_DelSig_StopConvert();
    
    return(ADCValue);
}

/*******************************************************************************
* Function Name: GetBatteryVoltage()
********************************************************************************
* Summary:
*  Reads the battery voltage
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/
void GetBatteryVoltage()
{   
    uint32 VoltageAcc=0;
    uint8 i=0;
    
    /* Temporarily disable the OC and OV protection logic */
    ControlReg_BlockComp_Write(0);

    /* Connect the Mux to read battery voltage */
    AMux_ADC_Select(BAT_VOLTAGE);  
    
    while(i<4)
    {
        /* Accumulate ADC readings */
        VoltageAcc+=ReadADC();
        i++;
    }
      
    /* Take average reading */
    VBat=VoltageAcc>>2;
    
    if(VBat < 0)
    {
        VBat = 0;   
    }
    
    /* Enable the OC and OV protection logic */
    ControlReg_BlockComp_Write(1);     
    
}

/*******************************************************************************
* Function Name: GetBatteryCurrent()
********************************************************************************
* Summary:
*  Reads the battery current
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/
void GetBatteryCurrent()
{

    uint32 CurrentAcc=0;
    uint8 i=0;
    
    /* Temporarily disable the OC and OV protection logic */
    ControlReg_BlockComp_Write(0);
    
    /* Connect the Mux to read battery charging current */
    AMux_ADC_Select(BAT_CURRENT);  
    
    while(i<4)
    {
        /* Accumulate ADC readings */
        CurrentAcc+=ReadADC();
        i++;
    }
    
    /* Take average reading */
    IBat=CurrentAcc>>2;        
    
    if(IBat < 0)
    {
        IBat = 0;
    }
    /* Enable the OC and OV protection logic */
    ControlReg_BlockComp_Write(1);
    

}

/*******************************************************************************
* Function Name: GetBatteryTemperature()
********************************************************************************
* Summary:
*  Reads the battery temperature
* 
*  Parameters:  None
*
*  Return: None
*******************************************************************************/

void GetBatteryTemperature()
{
	int32 iVhi, iVT;
    
     
    /* Set the ADC mux to read VDAC supplied voltage */
	AMux_ADC_Select(BAT_TEMP_1);  
    
    /* Read ADC */
     iVhi=ReadADC();
	 
    /* Set the ADC mux to read voltage across thermistor */ 
	 AMux_ADC_Select(BAT_TEMP_2);  
     
    /* Read ADC */
    iVT=ReadADC();
    
    /* Get the temperature value */
    TBat = Thermistor_Calc_1_GetTemperature((iVhi-iVT),iVT);
    
    /* Keep Mux connected to battery voltage */
    AMux_ADC_Select(BAT_VOLTAGE); 
}

/* [] END OF FILE */
