/*****************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*   The main C file for the BatteryCharger project. Contains an Constant-Current Constant- Voltage
*   based method to charge single cell Li-ion batteries.
*
******************************************************************************
* Copyright (C) 2012, Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/
#include <device.h>
#include "LiIonCharger.h"
#include "stdio.h"

/* USB debug enable */
#define USBUART_DEBUG	0x1

/* Variables required for USB debug */
uint8 DebugBuffer[16];
uint8 SendDebug;
uint8 Fault=0;

/*****************************************************************************
* Function Name: SystemInit()
******************************************************************************
* Summary:
* 	Initializes all componensts used. Does not exit loop till USB 
*   enumeration is complete
*
* Parameters:
*   none
*
* Return:
* 	none
*
* Side Effects:
*	none
*****************************************************************************/
void SystemInit()
{
    char8 buf[14];
    uint8 len;
    
    /* Start the LCD */ 
    LCD_Start();
    
    /* Start ADC */
    ADC_DelSig_Start();
        
    /* Start the charger */
    ChargerStart();  
    
    LCD_ClearDisplay();
    
    /* Display calculated scaling factor on LCD */
    LCD_Position(0,0);
    len=sprintf(buf,"Scale:%1.4f",Scale); 
    buf[len]='\0';
    LCD_PrintString(buf);

   
    CyDelay(2000);
    
    LCD_ClearDisplay();
    
    #if USBUART_DEBUG
    
	USBUART_Start(0, USBUART_3V_OPERATION);       /* Start USBFS Operation/device 0 and with 3V operation */
    while(!USBUART_bGetConfiguration());          /* Wait for Device to enumerate */
    USBUART_CDC_Init();
	Debug_ISR_Start();
	#endif
         
}

/*****************************************************************************
* Function Name: DebugDisplay()
******************************************************************************
* Summary:
* 	Puts data on Buffer and sends data to PC
*
* Parameters:
*   none
*
* Return:
* 	none
*
* Side Effects:
*	none
*****************************************************************************/

void DebugDisplay(void)
{
  
    
    #if USBUART_DEBUG
	
    DebugBuffer[0] = 0x0A;		 	/* Start Marker */
	DebugBuffer[1] = BatteryState;
	DebugBuffer[2] = ErrorState;    /* Error State */
    
    sprintf((char*)&DebugBuffer[3],"%04X%04X%04X",IBat,VBat,TBat); 
    DebugBuffer[15] = 0x0D;	 	    /* End marker */
	
    /*************************************/
	//USBUART_Write(DebugBuffer, 16);
	//while(!USBUART_bTxIsReady()){}
    /*************************************/
    
    while(USBUART_CDCIsReady() == 0u);       /* Wait till component is ready to send more data to the PC */ 
    USBUART_PutData(DebugBuffer, 16);        /* Send data back to PC */
                
	
    #endif
}

/*****************************************************************************
* Function Name: main()
******************************************************************************
* Summary:
* 	Main function of project. Runs loop on each Timer_Scan Trigger.
*   Displays parameters on LCD.
*
* Parameters:
*   none
*
* Return:
* 	none
*
* Side Effects:
*	none
*****************************************************************************/
void main()
{
    char8 buf[10];
    uint8 len;
    extern uint8 StatusFlags;
    CYGlobalIntEnable;
    
    /* Initialise components */
    SystemInit();
   
    for(;;)
    {
    
        /* StatusFlags is set in the ScanTime ISR. This loop is called every 10ms. */
        if(StatusFlags)
        {
            /* Reset the Flag */
            StatusFlags=0;
            
                     
            /* Check of OC/OV fault has occurred */
            if(Fault==0)
            {
                /*************************************************************/                
                /* Read out all the parameters */         
                GetBatteryVoltage();
                GetBatteryCurrent();                             
                GetBatteryTemperature();
                
                /* Update the state machine and take control action */
                UpdateBatteryState();            
                UpdateControlLoop();
                /*************************************************************/
                
                /*************************************************************/                
                /* Display the results on LCD */            
                LCD_Position(0,0);
                len=sprintf(buf,"I:%3.0fmA  ", IBat*0.3125);
                buf[len]='\0';
                
                LCD_PrintString(buf);
                
                LCD_Position(0,8);
                len=sprintf(buf,"V:%1.3fV", VBat*0.0000625/Scale);
                buf[len]='\0';
                LCD_PrintString(buf);
                
                LCD_Position(1,0);
                LCD_PrintHexUint16(PWMDuty);
                
                LCD_Position(1,8);
                len=sprintf(buf,"T:%2.1fC", TBat/100.0);
                buf[len]='\0';
                LCD_PrintString(buf);
                        
                /* Send debug information at 1 Hz. SendDebug Flag is set to 1 in Debug_ISR file */
                if(SendDebug)				
                {
                    DebugDisplay();
                    SendDebug = 0;
                }
                /*************************************************************/
                
            }
            else /* OC or OV Fault */
            {
                
                LCD_ClearDisplay();
                LCD_Position(0,0);
                LCD_PrintString("Fault");
                
                switch(ErrorState)
                {
                    case ER_START_TIMEOUT:
                       LCD_PrintString(": TO - INIT");
                       break;
                    
                    case ER_ACTIVE_TIMEOUT:
                       LCD_PrintString(": TO - ACT");
                       break;
                    
                    case ER_RAPID_TIMEOUT:
                       LCD_PrintString(": TO - RAP");
                       break;
                    
                    case ER_CV_TIMEOUT:
                       LCD_PrintString(": TO - CV");
                       break;
                    
                    case ER_TEMP_ERROR:
                       LCD_PrintString(": TEMP");
                       break;

                    case ER_OVER_VOLTAGE:
                       LCD_PrintString(": OV");
                       break;
                       
                    case ER_OVER_CURRENT:
                       LCD_PrintString(": OC");
                       break;
                       
                    default:
                    break;
                    
                }
                
                while(1);  /* Put a different code instead of halting here */
                
            }
        
        }
        
        /* code for other tasks */
        
    }
}


/* [] END OF FILE */
